/*
 * Gimbal
 * Copyright (C) 2024 moonleay
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package net.moonleay.gimbal.client.util.screen

import net.moonleay.gimbal.client.config.enums.HorizontalAnchor
import net.moonleay.gimbal.client.config.enums.VerticalAnchor

object ScreenUtil {

    fun getAnchor(x: Double, y: Double): Pair<HorizontalAnchor, VerticalAnchor> {
        return when (x) {
            in 0.0..<34.0 -> HorizontalAnchor.LEFT
            in 34.0..<67.0 -> HorizontalAnchor.CENTER
            else -> HorizontalAnchor.RIGHT
        } to when (y) {
            in 0.0..<34.0 -> VerticalAnchor.TOP
            in 34.0..<67.0 -> VerticalAnchor.CENTER
            else -> VerticalAnchor.BOTTOM
        }
    }

    fun getXForAnchor(x: Float, anchor: HorizontalAnchor, textWidth: Int): Float {
        return when (anchor) {
            HorizontalAnchor.LEFT -> x
            HorizontalAnchor.CENTER -> x - (textWidth / 2)
            HorizontalAnchor.RIGHT -> x - textWidth
        }
    }


    fun getYForAnchor(y: Float, anchor: VerticalAnchor, textHeight: Int): Float {
        return when (anchor) {
            VerticalAnchor.TOP -> y
            VerticalAnchor.CENTER -> y - (textHeight / 2)
            VerticalAnchor.BOTTOM -> y - textHeight
        }
    }

    fun getScaled(length: Int, real: Float): Double {
        val scale = length / 100.0
        return real / scale
    }

    fun getReal(length: Int, scaled: Double): Float {
        val scale = length / 100.0
        return (scaled * scale).toFloat()
    }

    fun isPositionOutOfBounds(x: Double, y: Double): Boolean {
        val tooBig = 100 < x || 100 < y
        val tooSmall = 0 > x || 0 > y
        return tooBig || tooSmall
    }

}
